Package: fansi
Title: ANSI Control Sequence Aware String Functions
Description: Counterparts to R string manipulation functions that account for
   the effects of ANSI text formatting control sequences.
Version: 1.0.7
Authors@R: c(
    person("Brodie", "Gaslam", email="brodie.gaslam@yahoo.com",
    role=c("aut", "cre")),
    person("Elliott", "Sales De Andrade", role="ctb"),
    person(given="R Core Team",
    email="R-core@r-project.org", role="cph",
    comment="UTF8 byte length calcs from src/util.c"
    ),
    person("Michael","Chirico", role="ctb",
    email="michaelchirico4@gmail.com", comment = c(ORCID="0000-0003-0787-087X")
    ),
    person(given = "Unicode, Inc.",
    role = c("cph", "dtc"),
    comment = "Unicode Character Database derivative data in src/width.c")
    )
Depends: R (>= 3.1.0)
License: GPL-2 | GPL-3
URL: https://github.com/brodieG/fansi
BugReports: https://github.com/brodieG/fansi/issues
VignetteBuilder: knitr
Suggests: unitizer, knitr, rmarkdown
Imports: grDevices, utils
RoxygenNote: 7.3.3
Encoding: UTF-8
Collate: 'constants.R' 'fansi-package.R' 'internal.R' 'load.R' 'misc.R'
        'nchar.R' 'strwrap.R' 'strtrim.R' 'strsplit.R' 'substr2.R'
        'trimws.R' 'tohtml.R' 'unhandled.R' 'normalize.R' 'sgr.R'
NeedsCompilation: yes
Packaged: 2025-11-18 23:27:14 UTC; brodie
Author: Brodie Gaslam [aut, cre],
  Elliott Sales De Andrade [ctb],
  R Core Team [cph] (UTF8 byte length calcs from src/util.c),
  Michael Chirico [ctb] (ORCID: <https://orcid.org/0000-0003-0787-087X>),
  Unicode, Inc. [cph, dtc] (Unicode Character Database derivative data in
    src/width.c)
Maintainer: Brodie Gaslam <brodie.gaslam@yahoo.com>
Repository: CRAN
Date/Publication: 2025-11-19 14:20:02 UTC
Built: R 4.6.0; aarch64-apple-darwin23; 2026-04-25 15:58:53 UTC; unix
Archs: fansi.so.dSYM
