## Copyright (C) Brodie Gaslam
##
## This file is part of "fansi - ANSI Control Sequence Aware String Functions"
##
## This program is free software: you can redistribute it and/or modify
## it under the terms of the GNU General Public License as published by
## the Free Software Foundation, either version 2 or 3 of the License.
##
## This program is distributed in the hope that it will be useful,
## but WITHOUT ANY WARRANTY; without even the implied warranty of
## MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
## GNU General Public License for more details.
##
## Go to <https://www.r-project.org/Licenses> for copies of the licenses.

# Test strings for use

end <- "\033[0m"
red <- "\033[31m"
inv <- "\033[7m"
grn.bg <- "\033[42m"
rgb.und <- "\033[4;38;2;0;120;200m"
rgb.und.256 <- "\033[4;38;5;141m"
rgb.256 <- "\033[48;5;141m"


